#include "rng.h"
#include "problem.h"
#include "ackley.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Ackley::Ackley(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Ackley::~Ackley(){
	free(transformedX);
	free(displacementVector);
}
	
double Ackley::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation		
		if(isShifted){
			for(int i=0;i<dimension;i++)
					transformedX[i]= x[i] - displacementVector[i];	
		}
		else{
			for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		}
	
		result = ackleyFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;			

		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;								
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;															
				printProgress();	
			}
		}		

		return result;
	}
	else	
		return oldResult;	
	
}

double Ackley::ackleyFunction(double* x){
	double acc1 = 0.0;
	double acc2 = 0.0;
	double oneoverd = 1.0/(double)dimension;

	for(int i=0;i<dimension;i++){
		acc1 += x[i]*x[i];
        	acc2 += cos(2.0*PI*x[i]);
	}
	acc1 = -0.2*sqrt(acc1*oneoverd);
    	acc2 *= oneoverd;

    	return  20.0 + exp(1) - 20.0*exp(acc1) - exp(acc2);
}

