#include "configuration.h"
#include <tclap/CmdLine.h>




using namespace TCLAP;
using namespace std;

	Configuration::Configuration(int argc,char** argv){
		try {  

			CmdLine cmd("Frankenstein's PSO", ' ', "1.0");
			
			//Required seed value
			ValueArg<unsigned long> seedArg("","seed","RNG seed",true,0,"unsigned long");
			cmd.add(seedArg);

			//Required maximum function evaluations value
			ValueArg<unsigned int> fesArg("","fes","Maximum number of function evaluations",true,0,"unsigned int");
			cmd.add(fesArg);
			
			//Required problem ID
			vector<unsigned int> allowedProblems;
			allowedProblems.push_back( ACKLEY );
			allowedProblems.push_back( GRIEWANK );
			allowedProblems.push_back( RASTRIGIN );
			allowedProblems.push_back( ROSENBROCK );	
			allowedProblems.push_back( EXTSCHAFFER );
			allowedProblems.push_back( SCHWEFEL );
			allowedProblems.push_back( SALOMON );
			allowedProblems.push_back( SPHERE );	
			allowedProblems.push_back( STEP );
			allowedProblems.push_back( WEIERSTRASS );	
			ValuesConstraint<unsigned int> allowedProblemIDVals( allowedProblems );
       
 			ValueArg<unsigned int> pIDArg("","problemID","Problem ID",true,0,&allowedProblemIDVals);
			cmd.add(pIDArg);

			//Required dimension value
 			ValueArg<unsigned int> dimArg("","dimensions","Dimensionality",true,0,"unsigned int");
			cmd.add(dimArg);

			//Optional flag. Default = randomly shifted
			SwitchArg randomShiftArg("","noshifted","Flag to switch off random shifting", true);
			cmd.add(randomShiftArg);				

			//Optional flag. Default = minimization
			SwitchArg minArg("","maximization","Flag to switch to maximization", true);
			cmd.add(minArg);
			
			//Required minimum value of the search range	
			ValueArg<double> minRangeArg("","minrange","Minimum value of the search range",true,0,"double");
			cmd.add(minRangeArg);

			//Required minimum value of the search range
			ValueArg<double> maxRangeArg("","maxrange","Maximum value of the search range",true,0,"double");
			cmd.add(maxRangeArg);
							
			//Required population size
			ValueArg<unsigned int> popArg("","popsize","Population size",true,0,"unsigned int");
			cmd.add(popArg);			

			//Required initial inertia weight
			ValueArg<double> initialIWArg("","initialIW","Initial inertia weight",true,1.0,"double");
			cmd.add(initialIWArg);
	
			//Required final inertia weight
			ValueArg<double> finalIWArg("","finalIW","Final inertia weight",true,0.0,"double");
			cmd.add(finalIWArg);

			//Required inertia weight schedule
			ValueArg<unsigned int> iwScheduleArg("","iwschedule","Inertia weight schedule",true,1,"unsigned int");
			cmd.add(iwScheduleArg);

			//Required topology schedule
			ValueArg<unsigned int> tScheduleArg("","tschedule","Topology schedule",true,1,"unsigned int");
			cmd.add(tScheduleArg); 			

			//Parses input
			cmd.parse( argc, argv );
	
			rngSeed = seedArg.getValue();
			maxFES = fesArg.getValue();

			problemID = pIDArg.getValue();
			problemDimension = dimArg.getValue();
			randomlyShifted=randomShiftArg.getValue();
			minimization = minArg.getValue();
			minInitRange=minRangeArg.getValue();
			maxInitRange=maxRangeArg.getValue();
			
			populationSize=popArg.getValue();		
			initialIW=initialIWArg.getValue();
			finalIW=finalIWArg.getValue();
			iwSchedule=iwScheduleArg.getValue();
			tSchedule=tScheduleArg.getValue();			

		}catch (TCLAP::ArgException &e)  // catch any exceptions
		{ std::cerr << "error: " << e.error() << " for arg " << e.argId() << std::endl; }
	}

	Configuration::~Configuration(){}

	unsigned long Configuration::getRNGSeed(){
		return rngSeed;
	}
 	unsigned int Configuration::getMaxFES(){
		return maxFES;
	}
	unsigned int Configuration::getProblemID(){
		return problemID;
	}
	unsigned int Configuration::getProblemDimension(){
		return problemDimension;
	}
	bool Configuration::isRandomlyShifted(){
		return randomlyShifted;
	}
	bool Configuration::isMinimization(){
		return minimization;
	}
	double Configuration::getMinInitRange(){
		return minInitRange;
	}
      	double Configuration::getMaxInitRange(){
		return maxInitRange;
	}	     
	unsigned int Configuration::getPopulationSize(){
		return populationSize;
	}
	double Configuration::getInitialIW(){
		return initialIW;	
	}
	double Configuration::getFinalIW(){
		return finalIW;	
	}
	unsigned int Configuration::getIWSchedule(){
		return iwSchedule;	
	}
	unsigned int Configuration::getTopologySchedule(){
		return tSchedule;
	}
	
	void Configuration::print(){
		cout << "Parameters:" << endl;
		cout << endl;
		cout << "rngSeed = " << rngSeed << endl;
		cout <<	"maxFES = " << maxFES << endl;
		cout << "problemID = " << problemID << endl;
		cout << "problemDimension = " << problemDimension << endl;
		cout << "randomlyShifted = " << randomlyShifted << endl; 
		cout << "minimization = " << minimization << endl;
		cout << "minInitRange = " << minInitRange << endl; 
		cout << "maxInitRange = " << maxInitRange << endl; 
		cout << "populationSize = " << populationSize << endl; 
		cout << "initialIW = " << initialIW << endl;
		cout << "finalIW = " << finalIW << endl;
		cout << "iwSchedule = " << iwSchedule << endl; 
		cout << "tSchedule =  " << tSchedule <<endl;		
	}
