#include "rng.h"
#include "problem.h"
#include "extschaffer.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

ExtendedSchaffer::ExtendedSchaffer(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

ExtendedSchaffer::~ExtendedSchaffer(){
	free(transformedX);
	free(displacementVector);
}
	
double ExtendedSchaffer::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation		
		if(isShifted){
			for(int i=0;i<dimension;i++)
					transformedX[i]= x[i] - displacementVector[i];	
		}
		else{
			for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		}
	
		result = extendedSchafferFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;												
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();	
			}
		}		
		
		return result;
	}
	else
		return oldResult;	
}

double ExtendedSchaffer::schafferFunction(double x,double y){
	return 0.5 + (pow(sin(pow(x, 2) + pow(y, 2)), 2) - 0.5) / pow(1 + 0.001 * (pow(x, 2) + pow(y, 2)),2);
}

double ExtendedSchaffer::extendedSchafferFunction(double* x){
	double acc =0.0;
	for(int i=0;i<dimension-1;i++)
		acc+=schafferFunction(x[i],x[i+1]);		
	acc+=schafferFunction(x[dimension-1],x[0]);		
	return acc;
}

