#include "rng.h"
#include "problem.h"
#include "griewank.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Griewank::Griewank(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Griewank::~Griewank(){
	free(transformedX);
	free(displacementVector);
}
	
double Griewank::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation		
		if(isShifted){
			for(int i=0;i<dimension;i++)
					transformedX[i]= x[i] - displacementVector[i];	
		}
		else{
			for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		}
	
		result = griewankFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;			
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();	
			}
		}		
		
		return result;
	}
	else
		return oldResult;	
}

double Griewank::griewankFunction(double* x){
	double acc1 = 0.0;
	double acc2 = 1.0;

	for(int i=0;i<dimension;i++){
		acc1 +=  x[i]*x[i];
		acc2 *= cos(x[i]/sqrt(i+1));
	}
	return acc1/4000.0 - acc2 +1;
	
}
