//Configuration
#include "configuration.h"

//Problems
#include "problem.h"

#include "ackley.h"
#include "griewank.h"
#include "rastrigin.h"
#include "rosenbrock.h"
#include "extschaffer.h"
#include "schwefel.h"
#include "salomon.h"
#include "sphere.h"
#include "step.h"
#include "weierstrass.h"

//Algorithms
#include "pso.h"
#include "frankenstein.h"

//C++ Libraries
#include <iostream>

using namespace std;

int main( int argc, char** argv) {
	Problem* problem;

	//Command line parsing
	Configuration* config = new Configuration(argc,argv);	  	    			

	//Random number generator
	RNG::initializeRNG(config->getRNGSeed());

	//Problem initialization
	switch(config->getProblemID()){
		case ACKLEY:{
			problem = new Ackley(config);
		}break;
		case GRIEWANK:{
			problem = new Griewank(config);
		}break;
		case RASTRIGIN:{
			problem = new Rastrigin(config);
		}break;
		case ROSENBROCK:{
			problem = new Rosenbrock(config);
		}break;
		case EXTSCHAFFER:{
			problem = new ExtendedSchaffer(config);
		}break;	
		case SCHWEFEL:{
			problem = new Schwefel(config);
		}break;
		case SALOMON:{
			problem = new Salomon(config);
		}break;
		case SPHERE:{
			problem = new Sphere(config);
		}break;
		case STEP:{
			problem = new Step(config);
		}break;
		case WEIERSTRASS:{
			problem = new Weierstrass(config);
		}break;
		default:{
			problem = new Sphere(config);
		}break;
	}

	//Swarm initialization
	Swarm* swarm = new Swarm(config,problem);

// 	swarm->print();

	//PSO intialization
	PSO* pso = new Frankenstein(config,swarm,problem);

	//PSO execution
	pso->run();	

// 	for(int i=0;i<config->getProblemDimension();i++){
// 		cout << (swarm->getSolution())[i] << " ";
// 	}
// 	cout << endl;

	//Memory release
	RNG::deallocateRNG();
	delete swarm;
	delete problem;
	delete pso;
	delete config;	
	
	return 0;
}
