#ifndef PARTICLE_H
#define PARTICLE_H

#include "rng.h"
#include "configuration.h"
#include "problem.h"

#include <vector>

class Particle {
  	
      protected:
	Configuration* config;
	Problem* problem;
	int id;
	
	double* currentPosition;	
	double* currentVelocity; 	
	double* bestPosition;
	double currentValue;	
	double bestValue;
	std::vector<int> neighbors;
		
      public:

	 Particle(Configuration* config,Problem* problem,int identifier);	
	 virtual ~Particle();

	int getID();

	int getDimension();

	std::vector<int> getNeighbors();
	unsigned int getNeighborhoodSize(); 
	void eraseNeighborhood();
	void addNeighbor(int neighborID);
	int getNeighborIDbyIndex(int neighborIndex);
	void eraseNeighborbyIndex(int neighborIndex);
	int getRandomNonAdjacentNeighborID();
	void eraseNeighborbyID(int nid);
		
	void evaluateCurrentPosition();	

	const double* getCurrentPosition();
	void setCurrentPosition(double* newPosition);
	
	const double* getCurrentVelocity();
	void setCurrentVelocity(double* newVelocity);

	const double* getBestPosition();
	void setBestPosition(double* newBestPosition);
	void updateBestPosition(double newBest);

	double getCurrentValue();
	void setCurrentValue(double newValue);

	double getBestValue();
	void setBestValue(double newBestValue);			

	void print();
};


#endif

