#include "rng.h"
#include "configuration.h"
#include "swarm.h"
#include "problem.h"
#include "pso.h"

PSO::PSO(Configuration* config, Swarm* swarm, Problem* problem){
	this->config = config;
	s = swarm;
	p = problem;	
}

PSO::~PSO(){}

double PSO::limitVelocity(double vel){
	if(vel > config->getMaxInitRange())
		return config->getMaxInitRange();
	else if(vel < config->getMinInitRange())
		return config->getMinInitRange();
	else
		return vel;
}

double PSO::clampPosition(double candidate){	
	if(candidate > config->getMaxInitRange()) 
		return config->getMaxInitRange();
	else if(candidate < config->getMinInitRange()) 
		return config->getMinInitRange();
	else
		return candidate;
}

double PSO::clampPositionVelocity(double candidate,double* velocity){	
	if(candidate > config->getMaxInitRange()) {
		*velocity = 0.0;
		return config->getMaxInitRange();
	}
	else if(candidate < config->getMinInitRange()) {
		*velocity = 0.0;
		return config->getMinInitRange();
	}
	else
		return candidate;
}

double PSO::reboundPosition(double candidate,double* velocity){
	double temp = candidate + *velocity;	
	if(temp > config->getMaxInitRange()) {		
		*velocity = -1*RNG::randVal(0.0,1.0)*(*velocity);
		if(*velocity > 0)					
			return config->getMaxInitRange() - *velocity;
		return config->getMaxInitRange() + *velocity;
	}
	else if(temp <  config->getMinInitRange()) {
		*velocity = -1*RNG::randVal(0.0,1.0)*(*velocity);
		if(*velocity < 0)					
			return  config->getMinInitRange() - *velocity;
		return  config->getMinInitRange() + *velocity;
	}
	else
		return temp;
}

void PSO::printSolution(){
	const double* solution = s->getSolution();
	for(unsigned int k=0;k<config->getProblemDimension();k++)
		printf("%lf ",solution[k]);
	printf("\n");
}

