#include "rng.h"
#include "problem.h"
#include "rastrigin.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Rastrigin::Rastrigin(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Rastrigin::~Rastrigin(){
	free(transformedX);
	free(displacementVector);
}
	
double Rastrigin::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation		
		if(isShifted){
			for(int i=0;i<dimension;i++)
					transformedX[i]= x[i] - displacementVector[i];	
		}
		else{
			for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		}
	
		result = rastriginFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

	if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;				
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();	
			}
		}		
		
		return result;
	}
	else
		return oldResult;	
}

double Rastrigin::rastriginFunction(double* x){	 
   	double res=0.0;
    	for (int i=0; i<dimension; i++)
        	res += (x[i]*x[i] - 10.0*cos(2.0*PI*x[i]));    	
    	return (res + 10.0*dimension);
}
