#include "rng.h"
#include "problem.h"
#include "rosenbrock.h"

#include <iostream>
#include <cstdlib>


using namespace std;

Rosenbrock::Rosenbrock(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Rosenbrock::~Rosenbrock(){
	free(transformedX);
	free(displacementVector);
}
	
double Rosenbrock::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation		
		if(isShifted){
			for(int i=0;i<dimension;i++)
					transformedX[i]= x[i] - displacementVector[i];	
		}
		else{
			for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		}
	
		result = rosenbrockFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;			
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();	
			}
		}		
		
		return result;
	}
	else
		return oldResult;	
}

double Rosenbrock::rosenbrockFunction(double* x){
	double acc =0.0;
	for(int i=0;i<dimension-1;i++)
		acc+=100.0*((x[i+1]-(x[i]*x[i]))*(x[i+1]-(x[i]*x[i])))+((x[i]-1.0)*(x[i]-1.0));
	return acc;
}
