#include "rng.h"
#include "problem.h"
#include "salomon.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Salomon::Salomon(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Salomon::~Salomon(){
	free(transformedX);
	free(displacementVector);
}
	
double Salomon::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation		
		if(isShifted){
			for(int i=0;i<dimension;i++)
					transformedX[i]= x[i] - displacementVector[i];	
		}
		else{
			for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		}
	
		result = salomonFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;					
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();	
			}
		}		

		return result;
	}
	else
		return oldResult;	
}

double Salomon::salomonFunction(double* x){	 
   	double acc=0.0;
    	for (int i=0; i<dimension; i++)
		acc+=x[i]*x[i];
	acc = sqrt(acc);
	return 1.0 - cos(2.0*PI*acc) + 0.1*acc;
        	
}
