#include "rng.h"
#include "problem.h"
#include "schwefel.h"

#include <iostream>
#include <cstdlib>
#include <cmath>
#include <cfloat>

using namespace std;

Schwefel::Schwefel(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Schwefel::~Schwefel(){
	free(transformedX);
	free(displacementVector);
}
	
double Schwefel::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation				
		for(int i=0;i<dimension;i++)
			transformedX[i]=x[i];
		
	
		result = schwefelFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;						
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();	
			}
		}		
		
		return result;
	}
	else
		return oldResult;	
}


double Schwefel::schwefelFunction(double* x){
	double sum=0.0;

	for (int i=0;i<dimension;i++){
		if(x[i]>500.0 || x[i]<-500.0)
			return DBL_MAX;
		else		
     			sum += x[i]*sin(sqrt(fabs(x[i])));
  	}

	return 418.9829*dimension - sum;
}

