#include "rng.h"
#include "problem.h"
#include "sphere.h"

#include <iostream>
#include <cstdlib>
#include <cmath>

using namespace std;

Sphere::Sphere(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Sphere::~Sphere(){
	free(transformedX);
	free(displacementVector);
}
	
double Sphere::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation		
		if(isShifted){
			for(int i=0;i<dimension;i++)
					transformedX[i]= x[i] - displacementVector[i];	
		}
		else{
			for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		}
	
		result = sphereFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;				
				printProgress();				
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();				
			}
		}		
		
		return result;
	}
	else
		return oldResult;	
}

double Sphere::sphereFunction(double* x){
	double acc =0.0;
	for(int i=0;i<dimension;i++)
		acc+=x[i]*x[i];
	return acc;
}
