#include "rng.h"
#include "problem.h"
#include "step.h"

#include <iostream>
#include <cstdlib>
#include <cmath>
#include <cfloat>

using namespace std;

Step::Step(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Step::~Step(){
	free(transformedX);
	free(displacementVector);
}
	
double Step::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation			
		for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		
	
		result = stepFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;					
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();	
			}
		}		
		
		return result;
	}
	else
		return oldResult;	
}

double Step::stepFunction(double* x){
	double acc1 = 0.0;	
	for(int i=0;i<dimension;i++){
		if(x[i]>5.12 || x[i]<-5.12)
			return DBL_MAX;
		else
			acc1 +=  floor(x[i]);	
	}
	
	return 6.0*dimension+acc1;	
}
