#include "rng.h"
#include "problem.h"
#include "weierstrass.h"

#include <iostream>
#include <cstdlib>
#include <cmath>
#include <cfloat>

using namespace std;

Weierstrass::Weierstrass(Configuration* config):Problem(config){
	optimum = 0.0;	

	transformedX = (double*)malloc(dimension*sizeof(double));
	displacementVector = (double*)malloc(dimension*sizeof(double));

// 	printf("Optimum at ");
	for(int i=0;i<dimension;i++){
 		displacementVector[i] = RNG::randVal(config->getMinInitRange(),config->getMaxInitRange());	 	
// 		displacementVector[i]  = 0.0;
//  		printf("%1.6lf ",displacementVector[i]);
	}
//  	printf("\n");
}

Weierstrass::~Weierstrass(){
	free(transformedX);
	free(displacementVector);
}
	
double Weierstrass::getFunctionValue(const double* x) {	
	double result;	

	if(evaluations < maxEvaluations){
				
		//Apply transformation		
		if(isShifted){
			for(int i=0;i<dimension;i++)
					transformedX[i]= x[i] - displacementVector[i];	
		}
		else{
			for(int i=0;i<dimension;i++)
					transformedX[i]=x[i];
		}
	
		result = weierstrassFunction(transformedX);
	
		//Update number of function evaluations
		evaluations++;
	
		oldResult  = result;	

		if(minimization){
			if( bestSolutionValue - result > DELTA){	
				bestSolutionValue = result;			
				printProgress();	
			}
		}
		else{
			if(result - bestSolutionValue > DELTA){
				bestSolutionValue = result;	
				printProgress();	
			}
		}		
		
		return result;
	}
	else
		return oldResult;	
}

double Weierstrass::weierstrassFunction(double* x){
	//parameters
	double a = 0.5;
	double b = 3.0;
	int k_max = 20;

	double acc0=0.0;
	for(int i=0;i<dimension;i++){
		if(x[i]>0.5 || x[i]<-0.5)
			return DBL_MAX;
		else {
			double acc1=0.0;	
			for(int j=0;j<k_max;j++)
				acc1+=(pow(a,j)*cos(2.0*PI*pow(b,j)*(x[i]+0.5)));
			acc0+=acc1;
		}
	}

	double acc2 =0.0;
	for(int i=0;i<k_max;i++)
		acc2+=(pow(a,i)*cos(PI*pow(b,i)));
	acc2*=dimension;


	return acc0-acc2;
}
